require 'test_helper'

module MerryGoRound
  class AggregatorTest < TestCase
    def test_aggregating_from_redis
      # Load the fixture. There is 60 minutes of data with 3 keys in each minute
      Timecop.freeze(Time.at(1360695600).utc) do
        redis_fixture('minutes')
        MerryGoRound.aggregate!
        MerryGoRound.aggregate!
      end

      # 60 minutes * 3 keys
      assert_equal 60 * 3, Aggregation.where(granularity: 'minute').count

      # 1 hour * 3 keys
      assert_equal 3, Aggregation.where(granularity: 'hour').count

      # The minute aggregations should belong to an hour one
      refute_nil Aggregation.where(granularity: 'minute').first.parent_id
      assert_equal 'hour', Aggregation.where(granularity: 'minute').first.parent.granularity
    end

    def test_compound_aggregations
      Timecop.freeze(Time.at(1360792056).utc) do
        csv_fixture('votes')
        MerryGoRound.aggregate!
        assert_equal 3, Aggregation.where(granularity: 'hour').count

        MerryGoRound.aggregate!
        assert_equal 3, Aggregation.where(granularity: 'hour').count
      end
    end
  end
end
