require 'test_helper'

module MerryGoRound
  class QueryTest < TestCase
    def test_execute
      Timecop.freeze(Time.at(1360695600).utc) do
        redis_fixture('minutes')
        MerryGoRound.aggregate!
      end

      min = MerryGoRound::Aggregation.minimum(:timestamp).to_i
      max = MerryGoRound::Aggregation.maximum(:timestamp).to_i

      query = Query.new(max: max, granularity: :hour)
      out = query.execute

      assert_equal 2, out.keys.length
      out[:results].keys.each do |key|
        assert_equal 1, out[:results][key].length
      end
    end

    def test_parsing_with_no_arguments
      Timecop.freeze(Time.now) do
        max_time = Time.at(Time.now.to_i)

        query = Query.new(keys: 'test')
        # should default to now, sans seconds
        expected_max = max_time.utc - max_time.utc.sec
        # should default to a day ago
        expected_min = expected_max - 86400

        assert_query_match query, expected_min, expected_max, 'test', :hour
      end
    end

    def test_parsing_with_min_and_max
      Timecop.freeze(Time.now) do
        # add some arbitrary offset
        max_time = Time.now - 5000
        # back in time by a day
        min_time = max_time - 86400

        query = Query.new(min: min_time.to_i, max: max_time.to_i, keys: 'test')
        expected_max = max_time.utc - max_time.utc.sec
        expected_min = min_time.utc - min_time.utc.sec

        assert_query_match query, expected_min, expected_max, 'test', :hour
      end
    end

    def test_parsing_with_granularity
      Timecop.freeze(Time.now) do
        max_time = Time.at(Time.now.to_i)

        query = Query.new(granularity: :week, keys: 'test')
        # should default to now, sans seconds
        expected_max = max_time.utc - max_time.utc.sec
        # should default to a day ago
        expected_min = expected_max - 31536000

        assert_query_match query, expected_min, expected_max, 'test', :week
      end
    end

    def assert_query_match(query, expected_min, expected_max, expected_key, expected_granularity)
      assert_equal expected_max.to_i, query.max.to_i
      assert_equal expected_min.to_i, query.min.to_i
      assert_equal expected_granularity, query.granularity
      assert_equal [expected_key], query.keys
    end
  end
end
